@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2012-2014 B.A.T.M.A.N. contributors:
+ * Copyright (C) 2012-2015  B.A.T.M.A.N. contributors:
  *
  * Simon Wunderlich
  *
@@ -20,11 +20,12 @@
  */
 
 #ifndef SOURCE_VERSION
-#define SOURCE_VERSION			"2014.4.0"
+#define SOURCE_VERSION			"2015.2"
 #endif
 
 #include <net/ethernet.h>
 #include <netinet/in.h>
+#include <netinet/udp.h>
 #include <stdint.h>
 #include <time.h>
 #include <sys/select.h>
@@ -55,6 +56,11 @@ struct dataset {
 	uint8_t local_data;
 };
 
+struct changed_data_type {
+	uint8_t data_type;
+	struct list_head list;
+};
+
 struct transaction_packet {
 	struct alfred_push_data_v0 *push;
 	struct list_head list;
@@ -97,6 +103,7 @@ struct interface {
 	uint32_t scope_id;
 	char *interface;
 	int netsock;
+	int netsock_mcast;
 
 	struct hashtable_t *server_hash;
 
@@ -113,10 +120,15 @@ struct globals {
 	enum clientmode clientmode;
 	int clientmode_arg;
 	int clientmode_version;
+	int verbose;
 
 	int unix_sock;
 	const char *unix_path;
 
+	const char *update_command;
+	struct list_head changed_data_types;
+	uint16_t changed_data_type_count; /* maximum is 256 */
+
 	struct timespec if_check;
 
 	struct hashtable_t *data_hash;
@@ -126,20 +138,23 @@ struct globals {
 #define debugMalloc(size, num)	malloc(size)
 #define debugFree(ptr, num)	free(ptr)
 
-#define MAX_PAYLOAD ((1 << 16) - 1)
+#define MAX_PAYLOAD ((1 << 16) - 1 - sizeof(struct udphdr))
 
 extern const struct in6_addr in6addr_localmcast;
 
 /* server.c */
 int alfred_server(struct globals *globals);
 int set_best_server(struct globals *globals);
+void changed_data_type(struct globals *globals, uint8_t arg);
+
 /* client.c */
 int alfred_client_request_data(struct globals *globals);
 int alfred_client_set_data(struct globals *globals);
 int alfred_client_modeswitch(struct globals *globals);
 int alfred_client_change_interface(struct globals *globals);
 /* recv.c */
-int recv_alfred_packet(struct globals *globals, struct interface *interface);
+int recv_alfred_packet(struct globals *globals, struct interface *interface,
+		       int recv_sock);
 struct transaction_head *
 transaction_add(struct globals *globals, struct ether_addr mac, uint16_t id);
 struct transaction_head *
